### ToDo:
# 1. Use list for categories (category$flood) rather than separate variables.


library(dplyr)

### console
options(max.print = 30)

### load files
filelist  <- list.files(path="./data/ready", pattern="\\.csv", full.names=TRUE, ignore.case=TRUE)
data <- do.call("rbind", lapply(filelist, function(f) {
  # f <- filelist[5]
  table <- read.csv(f,
    header = TRUE, quote = "\"",
    colClasses = c("character", "character", "numeric"),
    stringsAsFactors = FALSE
  )
  table$file <- gsub("^\\./data/ready/(.*)\\.csv$", "\\1", f)
  return (table)
}))
# for(i in 1:10) print(sprintf("[%1d] %.80s", data$label[i], data$tweet[i]))

# move reduplication to the dataset level
data <- subset(data, !duplicated(tweet))

### distrubute files to categories
disaster <- sort(unique(data$file))
# txt.header <- "category. <- c("
# txt.body   <- ""
# txt.footer <- ")"
# for(i in 1:length(disaster)) txt.body <- sprintf("%s\n  , %s", txt.body, disaster[i])
# cat(sprintf("%s%s\n%s", txt.header, txt.body, txt.footer))


category.wildfire <- c(
    "T26_2012_Colorado_wildfires"
  , "T26_2013_Australia_bushfire"
)

category.earthquake <- c(
    "T26_2012_Costa_Rica_earthquake"
  , "T26_2012_Guatemala_earthquake"
  , "T26_2012_Italy_earthquakes"
  , "T26_2013_Bohol_earthquake"
  , "AIDR2015Q2_2015_Nepal_Earthquake"
)

category.geophysical <- c(
    "AIDR2015Q2_Landslides_Worldwide"
  , "AIDR2015Q2_2014_Iceland_Volcano"
)

category.flood <- c(
    "T26_2012_Philipinnes_floods"
  , "T26_2013_Colorado_floods"
  , "T26_2013_Manila_floods"
  , "T26_2013_Sardinia_floods"
  , "T6_2013_Alberta_Floods"
  , "T6_2013_Queensland_Floods"
)

category.storm <- c(
  # include: cyclones, typhoons, hurricanes, tornado
    "T26_2012_Typhoon_Pablo"
  , "T26_2013_Typhoon_Yolanda"
  , "T6_2012_Sandy_Hurricane"
  , "T6_2013_Oklahoma_Tornado"
  , "AIDR2015Q2_2014_Arkansas_Tornado"
)

category.technologic <- c(
    "T26_2012_Venezuela_refinery"
  , "T26_2013_Brazil_nightclub_fire"
  , "T26_2013_Glasgow_helicopter_crash"
  , "T26_2013_Lac_Megantic_train_crash"
  , "T26_2013_NY_train_crash"
  , "T26_2013_Savar_building_collapse"
  , "T26_2013_Spain_train_crash"
  , "T6_2013_West_Texas_Explosion"
)

# category.terrorism <- c(
#     "T26_2013_LA_airport_shootings"
#   , "T6_2013_Boston_Bombings"
# )

# category.other <- c(
#   # out off category
#     "T26_2013_Russia_meteor"
#   , "T26_2013_Singapore_haze"
# )

# check we use all files
categories <- ls(pattern="category\\..*")
files_check <- Reduce(union, mget(categories))
files_check <- sort(files_check)
cat("Not used files:\n  ")
if (! isTRUE(all.equal(disaster, files_check))) {
  cat(sprintf("\n  %s", paste(setdiff(disaster, files_check), sep="\n")))
}

### write-back categories to the data
categories <- ls(pattern="category\\..*")
for (cat in categories) {
  data[
    which( data$label == 1 & data$file %in% get(cat) ),
    "category"
  ] <- gsub(".*\\.(.*)", "\\1", cat)
}

categories <- gsub("category\\.(.*)", "\\1", categories)
categories.size <- sapply(categories, function(cat) {
  nrow(subset(data, category == cat))
})
cat("\n\n")
print(categories.size)


### select on-topic records
balanced.on <- subset(data, (label == 1) & (category %in% categories))

# limit = 11000
# balanced.on <- subset(data, FALSE) # on-topic
# for (cat in categories) {
#   slice <- sample_n(
#     subset(data, (label == 1) & (category == cat)),
#     size = ifelse(categories.size[cat] < limit, categories.size[cat], limit),
#     replace = FALSE
#   )
#   balanced.on <- rbind(balanced.on, slice)
# }

### add off-topic records and permutate
balanced.off <- sample_n( # off-topic
  data[data$label == 0, ],
  size = nrow(balanced.on),
  replace = FALSE
)
balanced.off["file"] <- balanced.on["file"]
balanced <- rbind(balanced.on, balanced.off)
balanced <- balanced[sample(nrow(balanced), replace = FALSE), ]

### save ready-to-train-model dataset
# names(balanced)
# balanced["tweet.id"] <- NULL
saveRDS(balanced,  file="./saved/data.rds")



file.list <- sapply(unique(data$file), function(f) { nrow(data[data$file == f, ]) })
file.list <- data.frame(
  "file"  = names(file.list),
  "count" = file.list,
  row.names = NULL, stringsAsFactors=FALSE
)
file.list <- file.list[order(file.list$file), ]

for (cat in categories) {
  cat(sprintf("\n[%4d] %s",
    nrow(balanced.on[balanced.on$category == cat, ]),
    cat
  ))
  files <- unique(balanced.on[balanced.on$category == cat, ]$file)
  for (f in files) {
    cat(sprintf("\n    [%4d] %s",
      nrow(balanced.on[balanced.on$file == f, ]),
      f
    ))
  }
}


rm(cat, categories, categories.size, category.earthquake, category.flood,
  category.geophysical, category.storm, category.technologic, category.wildfire,
  disaster, f, filelist, files, files_check, file.list,
  balanced, balanced.off, balanced.on, data
)
