library(tm)
# library(SnowballC) # it seems last version of 'tm' forgot to set dependency

### console
options(max.print = 500)

data <- readRDS("./saved/data.rds") # see `01_04_balance.R`
# for(i in 1:10) cat(sprintf("\n[%1d] %.80s", data$label[i], data$tweet[i]))

### create corpus
data.corpus <- VCorpus(VectorSource(data$tweet), readerControl=list(language="english"))
# add metadata
meta(data.corpus, "label",    type="indexed") <- data$label
meta(data.corpus, "file",     type="indexed") <- data$file
meta(data.corpus, "tweed.id", type="indexed") <- data$tweet.id
# for(i in 1:10) cat(sprintf("\n[%d] %.80s", meta(data.corpus, type="index")$label[i], data.corpus[[i]]$content))

### cleaning
# ToDo: use `url` and `@user` as new features
data.lang <- meta(data.corpus[[1]], "language") # "english"

stoplist <- scan(file = "./data/stoplist.txt", what = "character", comment.char = "#")
stoplist <- unique(stoplist)

removeRegEx <- content_transformer( function(x, pattern) { gsub(pattern, "", x) } )
regex.user <- "(@[a-z0-9_]*)|(#[a-z0-9_]*)" # users and hash-tags
regex.url  <- "(?:https?://|www.|ftp.)[-a-z0-9+&@#/%=~_|$?!:,.…]*[a-z0-9+&@#/%=~_|$…]"
regex.html <- "&[a-z]+;"                    # html artefacts: "&amp;" "&lt;"

data.corpus <- tm_map(data.corpus, content_transformer(tolower), lazy=TRUE)
data.corpus <- tm_map(data.corpus, removeRegEx, regex.user)
data.corpus <- tm_map(data.corpus, removeRegEx, regex.url )
data.corpus <- tm_map(data.corpus, removeRegEx, regex.html)
data.corpus <- tm_map(data.corpus, removeNumbers, lazy=TRUE)
data.corpus <- tm_map(data.corpus, removePunctuation, lazy=TRUE)
data.corpus <- tm_map(data.corpus, removeWords, stopwords(data.lang), lazy=TRUE)
data.corpus <- tm_map(data.corpus, stemDocument, language=data.lang, lazy=TRUE)
data.corpus <- tm_map(data.corpus, removeWords, stoplist, lazy=TRUE) # remove specific dataset words
data.corpus <- tm_map(data.corpus, stripWhitespace)
data.corpus <- tm_map(data.corpus, removeRegEx, "(^\\s+)|(\\s+$)")   # leading & trailing space
data.corpus <- tm_filter(data.corpus, FUN = function(doc) (nchar(content(doc)) > 1))

for(i in 1:10) cat(sprintf("\n[%d] %.80s", meta(data.corpus, type="index")$label[i], data.corpus[[i]]$content))

### build vacabulary -----------------------------------------------------------
data.tdm <- TermDocumentMatrix(data.corpus, control = list(
  bounds = list(global = c(20, Inf))
))

### DocumentTermMatrix ---------------------------------------------------------
# ToDo: rewrite to use only dtm
# data.dtm <- DocumentTermMatrix(data.corpus, control=list(weighting=weightTf))
data.dtm <- t(data.tdm)

# remove empty documents
# rowTotals <- apply(data.dtm, 1, sum)
rowTotals <- slam::row_sums(data.dtm)
emptyRows <- data.dtm[rowTotals == 0, ]$dimnames[1][[1]]
data.corpus <- tm_filter(data.corpus, FUN = function(doc) {
  !is.element(meta(doc)$id, emptyRows) # (! meta(doc)$id %in% emptyRows )
})
data.dtm    <- data.dtm[rowTotals > 0, ]

saveRDS(data.corpus,             file="./saved/corpus.rds")
saveRDS(data.dtm,                file="./saved/dtmatrix.rds")
saveRDS(data.dtm$dimnames$Terms, file="./saved/vocabulary.rds")


rm(data, data.corpus, data.dtm, data.lang, data.tdm, emptyRows, i,
  regex.html, regex.url, regex.user, rowTotals, stoplist, removeRegEx
)

### Todo
# 1. convert @user, #tag to place-holders: %user%, %tag%
# 2. some hash-tags are real english words.
