library(ggplot2)
library(reshape2)
library(Cairo)

### console --------------------------------------------------------------------
options(max.print = 100)

### plot SVM -------------------------------------------------------------------
result <- read.csv("./saved/training_size_svm.csv", header = TRUE)

p <- ggplot(
  data = melt(result, id.vars = "size", measure.vars = c("prec", "rec", "auc")),
  mapping = aes_string(x = "size", y = "value")
)
p <- p + stat_summary(fun.data = "median_hilow", geom = "line",   aes_string(linetype = "variable"))
p <- p + stat_summary(fun.data = "median_hilow", geom = "ribbon", aes_string(fill = "variable"), alpha = 0.1, show.legend = FALSE)
p <- p + theme_bw(base_size = 11)
p <- p + scale_x_continuous(breaks = seq(from=1000, to=20000, by=2000))
p <- p + labs(title = "SVM model", x = "training dataset size", y = "model evaluation")
p <- p + scale_fill_grey(name = "Performance", start = 0.2, end = 0.2)
p <- p + scale_linetype_discrete(name = "Performance",
                        breaks = c("prec",      "rec",    "auc"),
                        labels = c("Precision", "Recall", "AUC"))
p <- p + theme(legend.position=c(.9, .2))
cat("PLOT: SVM model / training-ste size\n")
print(p)

### plot sLDA ------------------------------------------------------------------
result <- read.csv("./saved/training_size_slda.csv", header = TRUE)

p <- ggplot(
  data = melt(result, id.vars = "size", measure.vars = c("prec", "rec", "auc")),
  mapping = aes_string(x = "size", y = "value")
)
p <- p + stat_summary(fun.data = "median_hilow", geom = "line",   aes_string(linetype = "variable"))
p <- p + stat_summary(fun.data = "median_hilow", geom = "ribbon", aes_string(fill = "variable"), alpha = 0.1, show.legend = FALSE)
p <- p + theme_bw(base_size = 11)
p <- p + scale_x_continuous(breaks = seq(from=0, to=20000, by=2000))
p <- p + labs(title = "sLDA model", x = "training dataset size", y = "model evaluation")
p <- p + scale_fill_grey(name = "Performance", start = 0.2, end = 0.2)
p <- p + scale_linetype_discrete(name = "Performance",
                        breaks = c("prec",      "rec",    "auc"),
                        labels = c("Precision", "Recall", "AUC"))
p <- p + theme(legend.position=c(.9, .2))
cat("PLOT: sLDA model / training-ste size\n")
print(p)

### plot SVM & sLDA ------------------------------------------------------------
result1 <- read.csv("./saved/training_size_svm.csv",  header = TRUE)
result1 <- melt(result1, id.vars = "size", measure.vars = c("auc"))
result1$model <- "svm"
result2 <- read.csv("./saved/training_size_slda.csv", header = TRUE)
result2 <- melt(result2, id.vars = "size", measure.vars = c("auc"))
result2$model <- "slda"
result <- rbind(result1, result2)

p <- ggplot(
  data = result,
  mapping = aes_string(x = "size", y = "value", group = "model")
)
p <- p + stat_summary(fun.data = "median_hilow", geom = "line",   aes_string(linetype = "model"))
p <- p + stat_summary(fun.data = "median_hilow", geom = "ribbon", aes_string(fill = "model"), alpha = 0.1, show.legend = FALSE)
p <- p + theme_bw(base_size = 11)
p <- p + scale_x_continuous(breaks = seq(from=1000, to=20000, by=2000))
p <- p + labs(
  title = "SVM vs sLDA",
  x = "training dataset size",
  y = "model evaluation"
)
p <- p + scale_fill_grey(name = "Performance", start = 0.2, end = 0.2)
p <- p + scale_linetype_discrete(name = "Performance",
                        breaks = c("svm", "slda"),
                        labels = c("SVM", "sLDA"))
p <- p + theme(legend.position = c(.9, .2))
cat("PLOT: AUC of SVM & sLDA\n")
print(p)


### for publication ------------------------------------------------------------
# result <- read.csv("./saved/training_size_slda.csv", header = TRUE)
#
# p <- ggplot(
#   data = melt(result, id.vars = "size", measure.vars = c("prec", "rec")),
#   mapping = aes_string(x = "size", y = "value")
# )
# p <- p + stat_summary(fun.data = "median_hilow", geom = "line",   aes_string(linetype = "variable"))
# p <- p + stat_summary(fun.data = "median_hilow", geom = "ribbon", aes_string(fill = "variable"), alpha = 0.1, show.legend = FALSE)
# p <- p + theme_bw(base_size = 11)
# p <- p + scale_x_continuous(breaks = c(1000, 5000, 10000, 15000, 20000), expand = c(0, 0))
# p <- p + scale_y_continuous(limits = c(0.6, 0.9))
# p <- p + labs(x = "training dataset size", y = "model evaluation")
# p <- p + scale_fill_grey(start = 0.2, end = 0.2)
# p <- p + scale_linetype_discrete(name = "",
#                         breaks = c("prec",      "rec"),
#                         labels = c("Precision", "Recall"))
# p <- p + theme(legend.position=c(.8, .2))
# p
#
# pdf(
#   file = "./tmp/trainset-size.pdf",
#   width = 3.03, height = 2.5#, units = "in"
# )
# print(p)
# dev.off()


### ----------------------------------------------------------------------------

rm(result, result1, result2, p)
